;===================================================================================
    include \masm32\include\masm32rt.inc
    include \masm32\include\Iphlpapi.inc
    includelib \masm32\lib\Iphlpapi.lib
;===================================================================================

printf MACRO format:REQ, args:VARARG
    IFNB <args>
        invoke crt_printf, cfm$(format), args
    ELSE
        invoke crt_printf, cfm$(format)
    ENDIF
    EXITM <>
ENDM

;===================================================================================

comment |

//
// IP_ADDRESS_STRING - store an IP address as a dotted decimal string
//

typedef struct {
    char String[4 * 4];
} IP_ADDRESS_STRING, *PIP_ADDRESS_STRING, IP_MASK_STRING, *PIP_MASK_STRING;


//
// IP_ADDR_STRING - store an IP address with its corresponding subnet mask,
// both as dotted decimal strings
//

typedef struct _IP_ADDR_STRING {
    struct _IP_ADDR_STRING* Next;
    IP_ADDRESS_STRING IpAddress;
    IP_MASK_STRING IpMask;
    DWORD Context;
} IP_ADDR_STRING, *PIP_ADDR_STRING;

typedef struct _IP_ADAPTER_INFO {
  struct _IP_ADAPTER_INFO *Next;
  DWORD                   ComboIndex;
  char                    AdapterName[MAX_ADAPTER_NAME_LENGTH + 4];
  char                    Description[MAX_ADAPTER_DESCRIPTION_LENGTH + 4];
  UINT                    AddressLength;
  BYTE                    Address[MAX_ADAPTER_ADDRESS_LENGTH];
  DWORD                   Index;
  UINT                    Type;
  UINT                    DhcpEnabled;
  PIP_ADDR_STRING         CurrentIpAddress;
  IP_ADDR_STRING          IpAddressList;
  IP_ADDR_STRING          GatewayList;
  IP_ADDR_STRING          DhcpServer;
  BOOL                    HaveWins;
  IP_ADDR_STRING          PrimaryWinsServer;
  IP_ADDR_STRING          SecondaryWinsServer;
  time_t                  LeaseObtained;
  time_t                  LeaseExpires;
} IP_ADAPTER_INFO, *PIP_ADAPTER_INFO;
|

IP_ADDRESS_STRING STRUCT
    String BYTE 16 dup(?)
IP_ADDRESS_STRING ENDS

IP_MASK_STRING TYPEDEF IP_ADDRESS_STRING

IP_ADDR_STRING STRUCT
    Next      DWORD             ?
    IpAddress IP_ADDRESS_STRING <>
    IpMask    IP_MASK_STRING    <>
    Context   DWORD             ?
IP_ADDR_STRING ENDS

PIP_ADDR_STRING TYPEDEF PTR IP_ADDR_STRING

MAX_ADAPTER_DESCRIPTION_LENGTH  equ 128
MAX_ADAPTER_NAME_LENGTH         equ 256
MAX_ADAPTER_ADDRESS_LENGTH      equ 8

IP_ADAPTER_INFO STRUCT
    Next                DWORD ?
    ComboIndex          DWORD ?
    AdapterName         BYTE MAX_ADAPTER_NAME_LENGTH + 4 dup(?)
    Description         BYTE MAX_ADAPTER_DESCRIPTION_LENGTH + 4 dup(?)
    AddressLength       DWORD ?
    Address             BYTE MAX_ADAPTER_ADDRESS_LENGTH dup(?)
    Index               DWORD ?
    _Type               DWORD ?
    DhcpEnabled         DWORD ?
    CurrentIpAddress    PIP_ADDR_STRING ?
    IpAddressList       IP_ADDR_STRING <>
    GatewayList         IP_ADDR_STRING <>
    DhcpServer          IP_ADDR_STRING <>
    HaveWins            BOOL ?
    PrimaryWinsServer   IP_ADDR_STRING <>
    SecondaryWinsServer IP_ADDR_STRING <>
    LeaseObtained       DWORD ?
    LeaseExpires        DWORD ?
IP_ADAPTER_INFO ENDS

MIB_IF_TYPE_OTHER     equ 1
MIB_IF_TYPE_ETHERNET  equ 6
MIB_IF_TYPE_TOKENRING equ 9
MIB_IF_TYPE_FDDI      equ 15
MIB_IF_TYPE_PPP       equ 23
MIB_IF_TYPE_LOOPBACK  equ 24
MIB_IF_TYPE_SLIP      equ 28

;===================================================================================
    .data
        outBufLen dd SIZEOF IP_ADAPTER_INFO
    .code
;===================================================================================
start:
;===================================================================================

    printf("SIZEOF IP_ADAPTER_INFO: %d\n\n", outBufLen)

    mov ebx, alloc(outBufLen)
    .IF ebx == 0
        printf("Error allocating memory needed to call GetAdaptersinfo\n")
        inkey "Press any key to exit..."
        exit 1
    .ENDIF
    invoke GetAdaptersInfo, ebx, ADDR outBufLen
    .IF eax == ERROR_BUFFER_OVERFLOW
        free ebx
        mov ebx, alloc(outBufLen)
        .IF ebx == 0
            printf("Error allocating memory needed to call GetAdaptersinfo\n")
            inkey "Press any key to exit..."
            exit 1
        .ENDIF
    .ENDIF
    printf("Required output buffer length: %d\n\n", outBufLen)
    invoke GetAdaptersInfo, ebx, ADDR outBufLen
    .IF eax == NO_ERROR
        mov esi, ebx
        .WHILE esi

            printf("\tComboIndex: \t%d\n", [esi].IP_ADAPTER_INFO.ComboIndex)

            ;----------------------------------------------
            ; For %s printf expects a pointer to a string.
            ;----------------------------------------------

            mov eax, OFFSET IP_ADAPTER_INFO.AdapterName
            add eax, esi
            printf("\tAdapter Name: \t%s\n", eax)

            mov eax, OFFSET IP_ADAPTER_INFO.Description
            add eax, esi
            printf("\tAdapter Desc: \t%s\n", eax)

            printf("\tAdapter Addr: \t")
            mov edi, 1
            .WHILE edi < [esi].IP_ADAPTER_INFO.AddressLength
                movzx eax, BYTE PTR [esi+edi-1].IP_ADAPTER_INFO.Address
                printf("%.2X-", eax)
                inc edi
            .ENDW

            movzx eax, BYTE PTR [esi+edi-1].IP_ADAPTER_INFO.Address
            printf("%.2X\n", eax)

            printf("\tIndex: \t%d\n", [esi].IP_ADAPTER_INFO.Index)
            printf( "\tType: \t")
            SWITCH [esi].IP_ADAPTER_INFO._Type
                CASE MIB_IF_TYPE_OTHER
                    printf("Other\n")
                CASE MIB_IF_TYPE_ETHERNET
                    printf("Ethernet\n")
                CASE MIB_IF_TYPE_TOKENRING
                    printf("Token Ring\n")
                CASE MIB_IF_TYPE_FDDI
                    printf("FDDI\n")
                CASE MIB_IF_TYPE_PPP
                    printf("PPP\n")
                CASE MIB_IF_TYPE_LOOPBACK
                    printf("Lookback\n")
                CASE MIB_IF_TYPE_SLIP
                    printf("Slip\n")
                DEFAULT
                    printf("Unknown type %d\n", [esi].IP_ADAPTER_INFO._Type)
            ENDSW

            mov eax, OFFSET IP_ADAPTER_INFO.IpAddressList.IpAddress.String
            add eax, esi
            printf("\tIP Address: \t%s\n", eax)

            mov eax, OFFSET IP_ADAPTER_INFO.IpAddressList.IpMask.String
            add eax, esi
            printf("\tIP Mask: \t%s\n", eax)

            mov eax, OFFSET IP_ADAPTER_INFO.GatewayList.IpAddress.String
            add eax, esi
            printf("\tGateway: \t%s\n", eax)
            printf("\t***\n")

            .IF [esi].IP_ADAPTER_INFO.DhcpEnabled
                printf("\tDHCP Enabled: Yes\n")
                mov eax, OFFSET IP_ADAPTER_INFO.DhcpServer.IpAddress.String
                add eax, esi
                printf("\t  DHCP Server: \t%s\n", eax)
            .ELSE
                printf("\tDHCP Enabled: No\n")
            .ENDIF

            .IF [esi].IP_ADAPTER_INFO.HaveWins
                printf("\tHave Wins: Yes\n")
                mov eax, OFFSET IP_ADAPTER_INFO.PrimaryWinsServer.IpAddress.String
                add eax, esi
                printf("\t  Primary Wins Server:    %s\n", eax)
                mov eax, OFFSET IP_ADAPTER_INFO.SecondaryWinsServer.IpAddress.String
                add eax, esi
                printf("\t  Secondary Wins Server:    %s\n", eax)
            .ELSE
                printf("\tHave Wins: No\n")
            .ENDIF

            printf("\n")

            mov esi, [esi].IP_ADAPTER_INFO.Next

        .ENDW
    .ELSE
        printf("GetAdaptersInfo failed with error: %d\n", eax)
        inkey "Press any key to exit..."
        exit 1
    .ENDIF

    free ebx

    inkey "Press any key to exit..."
    exit 0
;===================================================================================
end start
